package com.fasga.rs.bot.deobfuscate;

import java.io.File;
import java.util.LinkedList;
import java.util.List;

import org.objectweb.asm.tree.ClassNode;

import com.fasga.rs.bot.Constants;

public abstract class Deobfuscator {
	private static List<Deobfuscator> deobfuscators = new LinkedList<Deobfuscator>();
	
	static {
		File directory = new File(Constants.DIRECTORY_DEOBFUSCATE);
		for(File file : directory.listFiles()) {
			if(!file.getName().equals("Deobfuscator.class")) {
				String name = file.getName().replaceAll(".class", "");
				try {
					Deobfuscator deobfuscator = (Deobfuscator) Class.forName(Constants.PACKAGE_DEOBFUSCATE + "." + name).newInstance();
					if(Deobfuscator.class.isAssignableFrom(deobfuscator.getClass())) {
						deobfuscators.add(deobfuscator);
					}
				} catch(InstantiationException e) {
					e.printStackTrace();
				} catch(IllegalAccessException e) {
					e.printStackTrace();
				} catch(ClassNotFoundException e) {
					e.printStackTrace();
				}
			}
		}
	}
	
	public abstract boolean applyDeobfuscation(ClassNode node);
	
	public static void run(ClassNode[] classes) {
		long start = System.currentTimeMillis();
		for(Deobfuscator deobfuscator : deobfuscators) {
			for(ClassNode node : classes) {
				deobfuscator.applyDeobfuscation(node);
			}
		}
		long end = System.currentTimeMillis();
		long time = end - start;
		System.out.println("Completed " + deobfuscators.size() + " deobfuscator(s) in " + time + "ms.");
	}
}
