package com.fasga.rs.bot.transform;

import java.util.Iterator;
import java.util.List;

import org.objectweb.asm.Opcodes;
import org.objectweb.asm.Type;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.FieldNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.MethodNode;

import com.fasga.rs.bot.util.TransformerHelper;

public class NodeChildTransformer extends Transformer {

	@Override
	public Class<?>[] getDependencies() {
		return new Class<?>[] { NodeTransformer.class };
	}

	@Override
	public boolean isApplicable(ClassNode node) {
		int numInstances = 0;
		String descriptor = Type.getObjectType(node.name).getDescriptor();
		if(node.superName.equals(classMap.get("Node").name)) {
			for(FieldNode field : (List<FieldNode>) node.fields) {
				if((field.access & Opcodes.ACC_STATIC) == 0) {
					if(field.desc.equals(descriptor)) {
						numInstances++;
					}
				}
			}
		}
		return numInstances == 2;
	}

	@Override
	public boolean transform(ClassNode node) {
		return hookNextAndPrevious(node);
	}


	private boolean hookNextAndPrevious(ClassNode node) {
		int count = 0;
		for(MethodNode method : (List<MethodNode>) node.methods) {
			if(!method.name.equals("<init>") && (method.access & Opcodes.ACC_STATIC) == 0 && method.desc.endsWith("V")) {
				InsnList list = method.instructions;
				Iterator<AbstractInsnNode> iterator = list.iterator();
				while(iterator.hasNext()) {
					AbstractInsnNode insn = iterator.next();
					if(insn.getOpcode() == Opcodes.GETFIELD) {
						FieldNode field = TransformerHelper.getFieldByName(node, ((FieldInsnNode) insn).name);
						switch(count++) {
							case 0:
								TransformerHelper.addGetter(node, field, "getNext", "Lcom/fasga/rs/bot/hook/NodeChild;");
								break;
							case 2:
								TransformerHelper.addGetter(node, field, "getPrevious", "Lcom/fasga/rs/bot/hook/NodeChild;");
								return true;
						}
					}
				}
			}
		}
		return false;
	}
}
