package com.fasga.rs.bot.transform;

import java.util.Iterator;
import java.util.List;

import org.objectweb.asm.Opcodes;
import org.objectweb.asm.tree.AbstractInsnNode;
import org.objectweb.asm.tree.ClassNode;
import org.objectweb.asm.tree.FieldInsnNode;
import org.objectweb.asm.tree.FieldNode;
import org.objectweb.asm.tree.InsnList;
import org.objectweb.asm.tree.MethodNode;

import com.fasga.rs.bot.Constants;
import com.fasga.rs.bot.util.TransformerHelper;

public class NodeTransformer extends Transformer {
	@Override
	public Class<? extends Transformer>[] getDependencies() {
		return Constants.NO_DEPENDENCIES;
	}

	@Override
	public boolean isApplicable(ClassNode node) {
		boolean hasLong = false;
		int count = 0;
		
		for(MethodNode method : (List<MethodNode>) node.methods) {
			if(method.name.equals("<init>") && method.desc.equals("()V")) {
				for(FieldNode field : (List<FieldNode>) node.fields) {
					if(field.desc.equals("J")) {
						hasLong = true;
					} else if(field.desc.equals("L" + node.name + ";")) {
						count++;
					}
				}
			}
		}
		return hasLong && count == 2;
	}

	@Override
	public boolean transform(ClassNode node) {
		return hookNextAndPrevious(node) && hookID(node);
	}
	
	private boolean hookNextAndPrevious(ClassNode node) {
		int count = 0;
		for(MethodNode method : (List<MethodNode>) node.methods) {
			if(!method.name.equals("<init>") && (method.access & Opcodes.ACC_STATIC) == 0 && method.desc.endsWith("V")) {
				InsnList list = method.instructions;
				Iterator<AbstractInsnNode> iterator = list.iterator();
				while(iterator.hasNext()) {
					AbstractInsnNode insn = iterator.next();
					if(insn.getOpcode() == Opcodes.GETFIELD) {
						FieldNode field = TransformerHelper.getFieldByName(node, ((FieldInsnNode) insn).name);
						switch(count++) {
							case 0:
								TransformerHelper.addGetter(node, field, "getNext", "Lcom/fasga/rs/bot/hook/Node;");
								break;
							case 2:
								TransformerHelper.addGetter(node, field, "getPrevious", "Lcom/fasga/rs/bot/hook/Node;");
								return true;
						}
					}
				}
			}
		}
		return false;
	}
	
	private boolean hookID(ClassNode node) {
		for(FieldNode field : (List<FieldNode>) node.fields) {
			if((field.access & Opcodes.ACC_STATIC) == 0 && field.desc.equals("J")) {
				TransformerHelper.addGetter(node, field, "getID", "J");
				return true;
			}
		}
		return false;
	}
}
