package com.fasga.rs.bot.transform;

import java.io.File;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.objectweb.asm.tree.ClassNode;

import com.fasga.rs.bot.Constants;
import com.fasga.rs.bot.util.TransformerHelper;

public abstract class Transformer {
	protected static Map<String, ClassNode> classMap = new HashMap<String, ClassNode>();
	
	private static List<Transformer> transformers = new LinkedList<Transformer>();

	private boolean hasRun = false;

	static {
		File directory = new File(Constants.DIRECTORY_TRANSFORM);
		for(File file : directory.listFiles()) {
			if(!file.getName().equals("Transformer.class")) {
				String name = file.getName().replaceAll(".class", "");
				try {
					Transformer transformer = (Transformer) Class.forName(Constants.PACKAGE_TRANSFORM + "." + name).newInstance();
					if(Transformer.class.isAssignableFrom(transformer.getClass())) {
						transformers.add(transformer);
					}
				} catch(InstantiationException e) {
					e.printStackTrace();
				} catch(IllegalAccessException e) {
					e.printStackTrace();
				} catch(ClassNotFoundException e) {
					e.printStackTrace();
				}
			}
		}
	}

	public abstract Class<?>[] getDependencies();

	public abstract boolean isApplicable(ClassNode node);

	public abstract boolean transform(ClassNode node);

	public static void run(ClassNode[] classes) {
		List<Transformer> trans = organize(transformers);
		int count = trans.size();
		long start = System.currentTimeMillis();
		for(int i = 0; i < trans.size(); i++) {
			Transformer transformer = trans.get(i);
			System.out.println("+ " + transformer.getClass().getSimpleName());
			for(ClassNode node : classes) {
				if(transformer.isApplicable(node)) {
					String idName = transformer.getClass().getSimpleName().replaceAll("Transformer", "");
					String hookName = Constants.PACKAGE_HOOK + "." + idName;
					classMap.put(idName, node);
					TransformerHelper.addInterface(node, hookName);
					if(!transformer.transform(node)) {
						System.err.println("\t[!!] Error in transformer \"" + transformer.getClass().getSimpleName() + "\"");
					}
				}
			}
			transformers.remove(transformer);
		}
		long end = System.currentTimeMillis();
		long time = end - start;
		System.out.println("Completed " + count + " transformer(s) in " + time + "ms.");
	}

	private static List<Transformer> organize(List<Transformer> before) {
		List<Transformer> after = new LinkedList<Transformer>();
		while(!before.isEmpty()) {
			for(int i = 0; i < before.size(); i++) {
				Transformer transformer = before.get(i);
				if(!after.contains(transformer)) {
					if(canRun(transformer)) {
						after.add(transformer);
						before.remove(transformer);
					}
				}
			}
		}
		return after;
	}

	private static boolean canRun(Transformer transformer) {
		Class<?>[] dependencies = transformer.getDependencies();
		for(Class<?> dependency : dependencies) {
			for(Transformer t : transformers) {
				if(t.getClass().equals(dependency)) {
					if(!t.hasRun) {
						return false;
					}
				}
			}
		}
		return true;
	}
}
